@@ -5,7 +5,8 @@ from pathlib import Path
 from sqlalchemy.orm import sessionmaker
 import pytest
 
-# See https://docs.sqlalchemy.org/en/14/orm/session_basics.html#when-do-i-make-a-sessionmaker # noqa: E501
+# See https://docs.sqlalchemy.org/en/14
+#      /orm/session_basics.html#when-do-i-make-a-sessionmaker
 # Tests should import from here to access the session
 #
 
@@ -15,7 +16,7 @@ TEST_DIR = os.path.dirname(os.path.abspath(__file__))
 # sqlite_db = Path(TEST_DIR, "/sqlite/test.db")
 # conn_string = f'sqlite://{sqlite_db}?check_same_thread=False'
 
-# con_string in memory version
+# con_string sqlite in memory version
 conn_string = "sqlite://?check_same_thread=False"
 
 Session = sessionmaker()
@@ -25,6 +26,6 @@ mhk_absent = pytest.mark.skipif(
 )
 
 skip_on_travis = pytest.mark.skipif(
-    os.environ.get("TRAVIS") == "true",
-    reason="this test required file system access for sqlite",
+    os.environ.get('TRAVIS') == 'true',
+    reason="this test requires file system access for sqlite"
 )
