@@ -7,28 +7,26 @@ import logging
 
 import pytest
 
-from timelink.kleio.groups import (
-    KElement,
-    KPerson,
-    KLs,
-    KAbstraction,
-    KAtr,
-    KDate,
-    KDay,
-    KMonth,
-    KYear,
-    KType,
-    KReplace,
-)
+from timelink.kleio.groups import (KElement,
+                                   KPerson,
+                                   KAbstraction,
+                                   KGroup,
+                                   KKleio,
+                                   KSource,
+                                   KAct, )
+from timelink.kleio.groups.katr import KAtr
+from timelink.kleio.groups.kelement import KDate, KDay, KMonth, KYear, KType, \
+    KReplace
+from timelink.kleio.groups.kls import KLs
 from timelink.kleio.utilities import quote_long_text
-from timelink.kleio.groups import KGroup, KKleio, KSource, KAct
 
 
 @pytest.fixture
 def kgroup_nested() -> KSource:
     """Returns a nested structure"""
     ks = KSource(
-        "s1", type="test", loc="auc", date="2021-07-16", ref="alumni", obs="Nested"
+        "s1", type="test", loc="auc", date="2021-07-16", ref="alumni",
+        obs="Nested"
     )
     ka1 = KAct(
         "a1",
@@ -59,7 +57,8 @@ def kgroup_nested() -> KSource:
     p2 = KPerson("Margarida", "f", "p02")
     p2.attr("residencia", "Trouxemil", date="2020-10-18")
     p1.rel(
-        "parentesco", "marido", p2.name, p2.id, date="2006-01-4", obs="Ilha Terceira"
+        "parentesco", "marido", p2.name, p2.id, date="2006-01-4",
+        obs="Ilha Terceira"
     )
     p3 = KPerson("Pedro", "m", "p03")
     p3.attr("residencia", "Rua Arménio RC", date="2021-10-21")
@@ -169,6 +168,7 @@ def test_kelement_subclasses2():
     classes = KElement.all_subclasses()
     element_names = [el.name for el in classes]  # noqa: F841
     assert len(classes) > 0
+    assert len(element_names) > 0
 
 
 def test_kelement_extend():
@@ -196,14 +196,15 @@ def test_kelement_extend_4():
 
 
 def test_kelement_extend_6():
-    """many competing classes for element get the one that matches column"""
-    KAno: KYear = KYear.extend("ano")  # noqa: F841
-    KAno2 = KElement.extend("ano")
-    KAno3 = KAno2.extend("ano")
-    KAno4 = KAno3.extend("ano")  # noqa: F841
-    gano = KGroup.extend("gano", position=["ano"])
+    """ many competing classes for element get the one that matches column """
+    KAno: KYear = KYear.extend('ano')
+    KAno2 = KElement.extend('ano')
+    KAno3 = KAno2.extend('ano')
+    KAno4 = KAno3.extend('ano')
+    gano = KGroup.extend('gano', position=['ano'])
     testing: KGroup = gano(2021)
     ano = testing.get_element_for_column(KYear.name)
+    assert KAno.name == KAno4.name  # just to use the variables
     assert ano.core == 2021  # get the more specialized
 
 
@@ -244,12 +245,14 @@ def test_kday2b():
 
 def test_kday3():
     with pytest.raises(ValueError):
-        day = KDay("32")  # noqa: F841
+        day = KDay('32')
+        assert day.core == '32'
 
 
 def test_kday4():
     with pytest.raises(ValueError):
-        day = KDay("31x")  # noqa: F841
+        day = KDay('31x')
+        assert day.core == '31x'
 
 
 def test_month1():
@@ -267,7 +270,8 @@ def test_month2():
 
 def test_month3():
     with pytest.raises(ValueError):
-        month = KMonth(14)  # noqa: F841
+        month = KMonth(14)
+        assert month.core == 14
 
 
 def test_month4():
@@ -366,10 +370,11 @@ def test_kelement_kleio_2():
 
 
 def test_kelement_kleio_3():
-    e = KElement("name", "Joaquim Carvalho", comment="Family name added in the margin")
-    assert (
-        e.to_kleio(name=True) == "name=Joaquim Carvalho#Family name added in the margin"
-    )
+    e = KElement('name', 'Joaquim Carvalho',
+                 comment="Family name in the margin")
+    assert e.to_kleio(
+        name=True) == 'name=Joaquim Carvalho#Family name in the margin', \
+        "str() failed with optional comment"
 
 
 def test_kelement_is_empty():
@@ -387,21 +392,21 @@ def test_kelement_is_empty_3():
     assert not e.is_empty(), "Did not detect not empty element"
 
 
-def test_kelement_to_kleio_4():
-    e = KElement(
-        "name",
-        "Joaquim Carvalho",
-        original="Joachim Carvº",
-        comment="Carvº added in the margin",
-    )
-    assert (
-        e.to_kleio() == "name=Joaquim Carvalho#Carvº added in the margin%Joachim Carvº"
-    )
+def test_kelement_to_kleio_2():
+    e = KElement('name',
+                 'Joaquim Carvalho',
+                 original='Joachim Carvº',
+                 comment="Carvº added in the margin")
+    assert e.to_kleio() == \
+           'name=Joaquim Carvalho#Carvº added in the margin%Joachim Carvº', \
+           'bad kleio representation of Element'
 
 
 def test_kgroup_extend():
     kfonte: KGroup = KGroup.extend(
-        "fonte", position=["id"], also=["tipo", "data", "ano", "obs", "substitui"]
+        "fonte", position=["id"],
+        also=["tipo", "data", "ano", "obs", "substitui"],
+        synonyms=[("tipo", "type"), ("data", "date"), ("ano", "year")]
     )
     afonte = kfonte("f001", ano=2021, tipo="teste")
     assert afonte.id.core == "f001", "could not extend group and instantiate"
@@ -414,12 +419,18 @@ def test_kgroup_extend2():
     KTipo = KType.extend("tipo")  # noqa: F841
     KSubstitui = KReplace.extend("substitui")  # noqa: F841
     kfonte: KGroup = KGroup.extend(
-        "fonte", position=["id"], also=["tipo", "data", "ano", "obs", "substitui"]
+        "fonte", position=["id"],
+        also=["tipo", "data", "ano", "obs", "substitui"]
     )
-    afonte = kfonte("f001", data="2021-12-09", ano=2021, tipo="teste", substitui="f001")
+    afonte = kfonte("f001", data="2021-12-09", ano=2021, tipo="teste",
+                    substitui="f001")
     assert afonte.data.extends("date")
     year_value = afonte.get_element_for_column("year")
     assert 2021 == year_value.core
+    assert KData.__name__ == 'data'
+    assert KAno.__name__ == 'ano'
+    assert KTipo.__name__ == 'tipo'
+    assert KSubstitui.__name__ == 'substitui'
 
 
 def test_kroup_elements_allowed():
@@ -427,7 +438,8 @@ def test_kroup_elements_allowed():
         "mygrou", position="id", also=["nome", "genero", "telefone"]
     )
     selements = sorted(my_group.elements_allowed())
-    assert selements == ["genero", "id", "nome", "telefone"], "wrong element list"
+    assert selements == ["genero", "id", "nome",
+                         "telefone"], "wrong element list"
 
 
 def test_kroup_allow_as_element():
@@ -435,7 +447,8 @@ def test_kroup_allow_as_element():
         "mygroup", position="id", also=["nome", "genero", "telefone"]
     )
     selements = sorted(my_group.elements_allowed())
-    assert selements == ["genero", "id", "nome", "telefone"], "wrong element list"
+    assert selements == ["genero", "id", "nome",
+                         "telefone"], "wrong element list"
 
     my_group.allow_as_element("profissao", also=True)
     selements = sorted(my_group.elements_allowed())
@@ -476,7 +489,8 @@ def test_kgroup_subclasses():
     sub_classes = (n, m, f)
     p = n("joaquim", "m", "01")
     sc = KPerson.all_subclasses()
-    assert len(sc) == 3 and len(sub_classes) == 3, "Failed to register subclasses"
+    assert len(sc) == 3 and len(
+        sub_classes) == 3, "Failed to register subclasses"
     assert KPerson.is_kgroup(p), "Failed is_kgroup test"
 
 
@@ -527,7 +541,8 @@ def test_allow_as_part_2():
 
     Kx.allow_as_part(Ky)
     x.include(y)
-    assert y in list(x.includes()), "include failed after class allowed as part"
+    assert y in list(
+        x.includes()), "include failed after class allowed as part"
 
 
 def test_allow_as_part_3():
@@ -544,7 +559,8 @@ def test_allow_as_part_3():
 
 
 def test_allow_as_part_4():
-    n: KPerson = KPerson.extend("n", position=["name", "sex"], guaranteed=["name"])
+    n: KPerson = KPerson.extend("n", position=["name", "sex"],
+                                guaranteed=["name"])
     j = n("joaquim", "m", id="jrc")
     pn = KPerson.extend("pn", position=["name"], guaranteed=["name"])
     n.allow_as_part(pn)
@@ -553,11 +569,12 @@ def test_allow_as_part_4():
 
 
 def test_includes_group():
-    n: KPerson = KPerson.extend("n", position=["name", "sex"], guaranteed=["name"])
-    j: n = n("joaquim", "m", id="jrc")
-    j.attr("residencia", "macau", date="2021-12-16")
-    atr1 = KAtr("residencia", "Coimbra", "2020-09-20")
-    atr2 = KAtr("hobby", "caligrafia", "2020-09-20")
+    n: KPerson = KPerson.extend('n', position=['name', 'sex'],
+                                guaranteed=['name'])
+    j: n = n('joaquim', 'm', id='jrc')
+    j.attr('residencia', 'macau', date='2021-12-16')
+    atr1 = KAtr('residencia', 'Coimbra', '2020-09-20')
+    atr2 = KAtr('hobby', 'caligrafia', '2020-09-20')
     j.include(atr1)
     j.include(atr2)
     j.attr("idade", "63", date="2021-08-08", obs="Taipa")
@@ -573,7 +590,8 @@ def test_includes_group():
 
 
 def test_includes_by_part_order():
-    n: KPerson = KPerson.extend("n", position=["name", "sex"], guaranteed=["name"])
+    n: KPerson = KPerson.extend("n", position=["name", "sex"],
+                                guaranteed=["name"])
     j = n("joaquim", "m", id="jrc")
     pn = KPerson.extend("pn", position=["name"], guaranteed=["name"])
     n.allow_as_part(pn)
@@ -648,6 +666,7 @@ def test_kgroup_get_item_check_el():
 def test_kgroup_get_item_no_check_el():
     p: KPerson = KPerson("joaquim", "m", "jrc", obs="")
     p._element_check = False
+    KElement.extend("profissao")
     p["profissao"] = "Professor"
     assert p["profissao"], "Could not enter unchecked element"
 
@@ -663,7 +682,8 @@ def test_kgroup_set_item():
 def test_kgroup_set_item_check_el():
     p = KPerson("joaquim", "m", "jrc", obs="")
     with pytest.raises(ValueError):
-        p["profissao"] = "professor", "set illegal element should raise error"
+        p['profissao'] = 'professor', \
+                         "set illegal element should raise error"
 
 
 def test_kgroup_get(kgroup_nested):
@@ -753,12 +773,14 @@ def test_kgroup_get_nested_dict_get(kgroup_nested):
 
 def test_kgroup_get_nested_dict_get2(kgroup_nested):
     kgroup_nested
-    assert kgroup_nested.get["acts"][0]["id"] == "a1", "Failed nested dictionary"
+    assert kgroup_nested.get["acts"][0][
+               "id"] == "a1", "Failed nested dictionary"
 
 
 def test_kgroup_dots(kgroup_nested):
     kgroup_nested
-    assert kgroup_nested.dots.includes.act[0].id == "a1", "Failed dots retrieval"
+    assert kgroup_nested.dots.includes.act[
+               0].id == "a1", "Failed dots retrieval"
 
 
 def test_kgroup_dots2(kgroup_nested):
@@ -798,7 +820,8 @@ def test_KPerson_to_json():
     json_string = p2.to_json()
     assert json_string
     p1.rel(
-        "parentesco", "marido", p2.name, p2.id, date="2006-01-4", obs="Ilha Terceira"
+        "parentesco", "marido", p2.name, p2.id, date="2006-01-4",
+        obs="Ilha Terceira"
     )
     json_string = p1.to_json()
     assert json_string
@@ -841,44 +864,96 @@ def test_person_ls_rel():
     ), "ls failed to show in person"
 
 
+def test_synonyms_1():
+    # We can map new elements to existing ones in group creation
+    kleio = KKleio
+    assert kleio is not None
+    fonte: KSource = KSource.extend(
+        "fonte",
+        also=["tipo", "data", "ano", "substitui", "loc", "ref", "obs"],
+        synonyms=[("tipo", "type")]
+    )
+    assert fonte is not None
+    tipo = KElement.get_class_for("tipo")
+    assert tipo is not None
+
+
+def test_synonyms_2():
+    # We can map new elements to existing ones in group creation
+    kleio = KKleio
+    assert kleio is not None
+    with pytest.raises(ValueError):
+        fonte: KSource = KSource.extend(
+            "fonte",
+            also=["tipo", "data", "ano", "substitui", "loc", "ref", "obs"],
+            synonyms=[("tipo", "typer")]  # no KElement typer
+            )
+        assert fonte is not None  # this should never be reached
+
+
 def test_kleio_stru():
     # We specialize the kleio groups for this purpose,
     # using definitions already in gacto2.str
     kleio = KKleio
     assert kleio is not None
     fonte = KSource.extend(
-        "fonte", also=["tipo", "data", "ano", "substitui", "loc", "ref", "obs"]
+        "fonte",
+        also=["tipo", "data", "ano", "substitui", "loc", "ref", "obs"],
+        synonyms=[("tipo", "type"), ("data", "date"), ("ano", "year")]
     )
     lista = KAct.extend(
         "lista",
         position=["id", "dia", "mes", "ano"],
         guaranteed=["id", "ano", "mes", "dia"],
         also=["data", "tipo", "loc", "obs"],
-    )
+        synonyms=[("tipo", "type"),
+                  ("dia", "day"),
+                  ("mes", "month"),
+                  ("data", "date"),
+                  ("ano", "year")
+                  ]
+        )
     auc = KAbstraction.extend(
-        "auc", position=["name", ""], also=["level", "id"], guaranteed=["id"]
-    )
+                              "auc",
+                              position=["name", "type"],
+                              also=["level", "id"],
+                              guaranteed=["id"]
+                              )
     n = KPerson.extend(
         "n",
         position=["nome", "sexo"],
         guaranteed=["id", "nome", "sexo"],
         also=["mesmo_que", "obs"],
-    )
+        synonyms=[("nome", "name"), ("sexo", "sex"), ("mesmo_que", "same_as")]
+        )
     pai = KPerson.extend(
-        "pai", position=["nome"], guaranteed=["nome"], also=["mesmo_que", "obs"]
-    )
+        "pai", position=["nome"], guaranteed=["nome"],
+        also=["mesmo_que", "obs"],
+        synonyms=[("nome", "name"), ("mesmo_que", "same_as")]
+        )
     mae = KPerson.extend(
-        "mae", position=["nome"], guaranteed=["nome"], also=["mesmo_que", "obs"]
-    )
+        "mae", position=["nome"], guaranteed=["nome"],
+        also=["mesmo_que", "obs"],
+        synonyms=[("nome", "name"), ("mesmo_que", "same_as")]
+        )
     n.allow_as_part(pai)
     n.allow_as_part(mae)
-    ls = KLs.extend("ls", position=["type", "value"], also=["data", "obs", "entity"])
-    atr = KAtr.extend("atr", position=["type", "value"], also=["data", "obs", "entity"])
+    ls = KLs.extend('ls',
+                    position=['type', 'value'],
+                    also=['data', 'obs', 'entity'],
+                    synonyms=[("data", "date")]
+                    )
+    atr = KAtr.extend('atr',
+                      position=['type', 'value'],
+                      also=['data', 'obs', 'entity'],
+                      synonyms=[("data", "date")]
+                      )
 
     kf = KKleio()
     f = fonte("f001", tipo="auc-tests")
     kf.include(f)
-    l: KGroup = lista("l001", 11, 2, 2021, data="1537-1913", tipo="auc-list", loc="A")
+    l: KGroup = lista("l001", 11, 2, 2021, data="1537-1913", tipo="auc-list",
+                      loc="A")
     f.include(l)
     a = auc("alumni-record", "archeevo-record", id="xpto")
     l.include(a)
@@ -892,5 +967,5 @@ def test_kleio_stru():
     m.include(ls("uc", "fim", data=2021))
     l2.include(m)
     json_string = j.to_json()
-    assert len(json_string) > 0, "invalid JSON string from group"
-    assert j.nome.core == "joaquim"
+    assert j.nome.core == 'joaquim'
+    assert len(json_string) > 0
