@@ -3,9 +3,10 @@
 """Tests for `timelink-py` package."""
 
 import pytest
-from click.testing import CliRunner
+from typer.testing import CliRunner
 
-from timelink import cli
+from timelink.cli import app
+from tests import mhk_absent
 
 
 @pytest.fixture
@@ -27,9 +28,25 @@ def test_content(response):
 def test_command_line_interface():
     """Test the CLI."""
     runner = CliRunner()
-    result = runner.invoke(cli.cli)
+    result = runner.invoke(app, ["--help"])
     assert result.exit_code == 0
-    assert 'timelink  timelink command line interface' in result.output
-    help_result = runner.invoke(cli.cli, ['--help'])
-    assert help_result.exit_code == 0
-    assert '--help  Show this message and exit.' in help_result.output
+    assert "Timelink and MHK manager" in result.output
+
+
+@mhk_absent
+def test_command_mhk_version():
+    """Test the CLI."""
+    runner = CliRunner()
+    result = runner.invoke(app, ["mhk", "version"])
+    print(f"Test result {result.exit_code}")
+    # assert result.exit_code == 0
+    assert "Portainer URL" in result.output or "not found" in result.output
+
+
+@mhk_absent
+def test_command_mhk_status():
+    """Test the CLI."""
+    runner = CliRunner()
+    result = runner.invoke(app, ["mhk", "status"])
+    assert result.exit_code == 0
+    assert "Stopped" in result.output
