@@ -1,17 +1,21 @@
 from sqlalchemy import Column, String, ForeignKey
 
-from timelink.mhk.models.base import Entity
+from timelink.mhk.models.entity import Entity
 
 
 class Person(Entity):
     __tablename__ = "persons"
 
+    id = Column(String, ForeignKey('entities.id'), primary_key=True)
+    name = Column(String, index=True)
     id = Column(String, ForeignKey("entities.id"), primary_key=True)
     name = Column(String)
     sex = Column(String(1))
     obs = Column(String)
 
-    __mapper_args__ = {"polymorphic_identity": "person"}
+    __mapper_args__ = {
+        'polymorphic_identity': 'person'
+    }
 
     def __repr__(self):
         sr = super().__repr__()
